################################### FUNCTIONS ###################################

# Read multiple featureCounts output matrices in a single dataset for DESeq2 analysis (from biostars, modified DESeqDatasetFromHTSeqCount)
DESeqDataSetFromFeatureCounts <- function (sampleTable, directory = ".", design, ignoreRank = FALSE, ...) 
{
  if (missing(design)) 
    stop("design is missing")
  l <- lapply(as.character(sampleTable[, 2]), function(fn) read.table(file.path(directory, fn), skip=2))
  if (!all(sapply(l, function(a) all(a$V1 == l[[1]]$V1)))) 
    stop("Gene IDs (first column) differ between files.")
  tbl <- sapply(l, function(a) a$V7)
  colnames(tbl) <- sampleTable[, 1]
  rownames(tbl) <- l[[1]]$V1
  rownames(sampleTable) <- sampleTable[, 1]
  dds <- DESeqDataSetFromMatrix(countData = tbl, colData = sampleTable[, 
                                                                       -(1:2), drop = FALSE], design = design, ignoreRank, ...)
  return(dds)
}

# Create SampleTable for DESeq2
createSampleTable <- function(sampleFiles, clinTable) {
  sampleFiles = sampleFiles[match(sub(".bam", "\\1", clinTable$bam), sampleFiles)]
  stopifnot(all(sampleFiles == sub(".bam", "\\1", clinTable$bam)))
  SmokingStatus <- clinTable$SmokingStatus2
  SmokingStatus1 <- clinTable$SmokingStatus
  Batch <- clinTable$Experiment
  Gender <- clinTable$Gender
  Age <- clinTable$Age
  COPD <- clinTable$COPD2
  PacksYear <- clinTable$PacksYear
  CancerType <- clinTable$CancerType
  Cancer <- clinTable$Cancer
  Tissue <- clinTable$Tissue
  Hv <- ifelse(clinTable$CancerType=='HV', 'Healthy', 'Clinic')
  sampleTable <- data.frame(sampleName = rownames(clinTable),
                            fileName = sampleFiles,
                            batch=Batch,
                            sex=Gender,
                            age=Age,
                            COPD=COPD,
                            smokingStatus=SmokingStatus,
                            smokingStatus1=SmokingStatus1,
                            packyears=PacksYear,
                            cancerType=CancerType,
                            cancer=Cancer,
                            tissue=Tissue,
                            hv=Hv,
                            stringsAsFactors = T)
  sampleTable$batch <- droplevels(sampleTable$batch)
  sampleTable$sex <- droplevels(sampleTable$sex)
  sampleTable$age <- droplevels(sampleTable$age)
  sampleTable$COPD <- droplevels(sampleTable$COPD)
  sampleTable$smokingStatus <- droplevels(sampleTable$smokingStatus)
  sampleTable$packyears <- droplevels(sampleTable$packyears)
  sampleTable$cancerType <- droplevels(sampleTable$cancerType)
  sampleTable$cancer <- droplevels(sampleTable$cancer)
  sampleTable$tissue <- droplevels(sampleTable$tissue)
  sampleTable$smokingStatus1 <- droplevels(sampleTable$smokingStatus1)
  
  return(sampleTable)
}

################################### LIBRARIES ###################################

library(ggfortify)
library(DESeq2)

################################### DATA NORMALIZATION ###################################

# read clinical table
clinical <- readRDS("data/clinical/full_clinical_table.RDS")

# read expression data (all samples)
raw_counts_all <- read.table("data/raw_counts/raw_counts_allsamples.tsv", sep = '\t', header = T) # table containing raw counts from feature counts for all samples

# normalize counts with DESeq2 vst
countTableDir <- "data/RNA_BAM_v2.0_cleaned_rgs_reordered_count_tables_protein_coding/" # directory containing feature counts output: one file for each sample containing raw counts for all protein-coding genes
sampleFiles <- list.files(countTableDir)
sampleTable <- createSampleTable(sampleFiles, clinical)
DDS_all_samples <- DESeqDataSetFromFeatureCounts(sampleTable = sampleTable,
                                        directory = countTableDir,
                                        design= ~ batch)

# Make nasal expression matrix
DDS_ns_vst <- vst(DDS_all_samples[,DDS_all_samples$tissue=='Nasal'], blind = T) # blind the transformation to the experimental design
vst_counts_ns <- t(assay(DDS_ns_vst))
# add clinical and technical variables
vst_counts_ns <- cbind(vst_counts_ns, clinical[clinical$Tissue=='Nasal',])
vst_counts_ns$hv_status <- ifelse(vst_counts_ns$CancerType=='HV', 'HV', 'Symptomatic')

write.table(vst_counts_ns,file="data/normalized_counts/exprDataAnnotatedNS.tsv",sep="\t",quote=FALSE,row.names=FALSE)

# Make bronchial expression matrix
DDS_br_vst <- vst(DDS_all_samples[,DDS_all_samples$tissue=='Bronchial'], blind = T)
vst_counts_br <- t(assay(DDS_br_vst))
# add clinical and technical variables
vst_counts_br <- cbind(vst_counts_br, clinical[clinical$Tissue=='Nasal',])
vst_counts_br$hv_status <- ifelse(vst_counts_br$CancerType=='HV', 'HV', 'Symptomatic')

write.table(vst_counts_ns,file="data/normalized_counts/exprDataAnnotatedBR.tsv",sep="\t",quote=FALSE,row.names=FALSE)
