################################### FUNCTIONS ###################################

# Read multiple featureCounts output matrices in a single dataset for DESeq2 analysis (from biostars, modified DESeqDatasetFromHTSeqCount)
DESeqDataSetFromFeatureCounts <- function (sampleTable, directory = ".", design, ignoreRank = FALSE, ...) 
{
  if (missing(design)) 
    stop("design is missing")
  l <- lapply(as.character(sampleTable[, 2]), function(fn) read.table(file.path(directory, fn), skip=2))
  if (!all(sapply(l, function(a) all(a$V1 == l[[1]]$V1)))) 
    stop("Gene IDs (first column) differ between files.")
  tbl <- sapply(l, function(a) a$V7)
  colnames(tbl) <- sampleTable[, 1]
  rownames(tbl) <- l[[1]]$V1
  rownames(sampleTable) <- sampleTable[, 1]
  dds <- DESeqDataSetFromMatrix(countData = tbl, colData = sampleTable[, 
                                                                       -(1:2), drop = FALSE], design = design, ignoreRank, ...)
  return(dds)
}

# Calculate TPM from raw counts
tpm <- function(counts, lengths) {
  rate <- counts / lengths
  rate / sum(rate) * 1e6
}

# Create SampleTable for DESeq2
createSampleTable <- function(sampleFiles, clinTable) {
  sampleFiles = sampleFiles[match(sub(".bam", "\\1", clinTable$bam), sampleFiles)]
  stopifnot(all(sampleFiles == sub(".bam", "\\1", clinTable$bam)))
  SmokingStatus <- clinTable$SmokingStatus2
  SmokingStatus1 <- clinTable$SmokingStatus
  Batch <- clinTable$Experiment
  Gender <- clinTable$Gender
  Age <- clinTable$Age
  COPD <- clinTable$COPD2
  PacksYear <- clinTable$PacksYear
  CancerType <- clinTable$CancerType
  Cancer <- clinTable$Cancer
  Tissue <- clinTable$Tissue
  Hv <- ifelse(clinTable$CancerType=='HV', 'Healthy', 'Clinic')
  sampleTable <- data.frame(sampleName = rownames(clinTable),
                            fileName = sampleFiles,
                            batch=Batch,
                            sex=Gender,
                            age=Age,
                            COPD=COPD,
                            smokingStatus=SmokingStatus,
                            smokingStatus1=SmokingStatus1,
                            packyears=PacksYear,
                            cancerType=CancerType,
                            cancer=Cancer,
                            tissue=Tissue,
                            hv=Hv,
                            stringsAsFactors = T)
  sampleTable$batch <- droplevels(sampleTable$batch)
  sampleTable$sex <- droplevels(sampleTable$sex)
  sampleTable$age <- droplevels(sampleTable$age)
  sampleTable$COPD <- droplevels(sampleTable$COPD)
  sampleTable$smokingStatus <- droplevels(sampleTable$smokingStatus)
  sampleTable$packyears <- droplevels(sampleTable$packyears)
  sampleTable$cancerType <- droplevels(sampleTable$cancerType)
  sampleTable$cancer <- droplevels(sampleTable$cancer)
  sampleTable$tissue <- droplevels(sampleTable$tissue)
  sampleTable$smokingStatus1 <- droplevels(sampleTable$smokingStatus1)
  
  return(sampleTable)
}

runDeseq <- function(file_list, clinical_table, contrast_variable, ref_condition, tissue=NULL, smoking_categories=NULL, confounding=NULL, donor_group=c('healthy','symptomatic'), removeIneligible=FALSE){
  clin <- clinical_table
  if(!is.null(tissue)){
    clin <- clin[clin$Tissue == tissue,]
  }
  clin$Tissue <- droplevels(clin$Tissue)
  
  if(!is.null(smoking_categories)){
    clin <- clin[clin$SmokingStatus %in% smoking_categories,]
  }
  clin$SmokingStatus <- droplevels(clin$SmokingStatus)
  
  if(removeIneligible){
    clin <- clin[clin$CancerType != 'Ineligible',]
  }
  clin$CancerType <- droplevels(clin$CancerType)
  
  # Create sampleTable for DESeq2
  sampleTable <- createSampleTable(file_list, clin)
  
  sampleTable <- sampleTable[sampleTable$hv %in% donor_group,]
  sampleTable$hv <- droplevels(sampleTable$hv)
  
  sampleTable <- sampleTable[!is.na(sampleTable$packyears),]
  
  print(table(sampleTable$cancer, sampleTable$hv))
  
  form <- as.formula(paste(paste( ' ~ ', confounding), contrast_variable, sep = ' + '))
  
  # Create DESeq2 DDS object
  print('Creating DDS object...')
  dds <- DESeqDataSetFromFeatureCounts(sampleTable = sampleTable,
                                       directory = countTableDir,
                                       design= form)
  # pre-filtering to reduce the size of dds
  keep <- rowSums(counts(dds)) >= 10
  dds <- dds[keep,]
  # set control level
  colData(dds)[,contrast_variable] <- relevel(colData(dds)[,contrast_variable] ,ref = ref_condition)
  # run DESeq and extract results
  print('Running DESeq...')
  dds <- DESeq(dds, betaPrior = FALSE)
  res <- results(dds)
  output <- list(dds_object=dds, result_table=res)
}


################################### LOAD LIBRARIES ###################################
library(DESeq2)
library(clusterProfiler)
library(variancePartition)
library(tidyr)

################################### DIFFERENTIAL EXPRESSION ###################################

# Read-in clinical information
clin_all <- readRDS("data/clinical/full_clinical_table.RDS") # this is the metadata table containing clinical annotation for all patients/samples

# Read-in count tables
countTableDir <- "data/raw_counts/" # directory containing feature counts output: one file for each sample containing raw counts for all protein-coding genes
sampleFiles <- list.files(countTableDir)

# Differential expression between clinic and healthy subjects
results_healthyVsClinic <- runDeseq(file_list = sampleFiles,
         clinical_table = clin_all,
         contrast_variable = 'hv',
         ref_condition = 'Healthy',
         tissue='Nasal',
         smoking_categories=c('current','ex.less1month', 'ex.1_to_12months', 'ex.more12months'),
         confounding='age + batch + sex + smokingStatus + packyears',
         donor_group=c('Healthy','Clinic'),
         removeIneligible=TRUE)

# Differential expression between clinic cancer and clinic benign in nasal tissue
results_nasalCancerVsNoCancer <- runDeseq(file_list = sampleFiles,
                                    clinical_table = clin_all,
                                    contrast_variable = 'cancer',
                                    ref_condition = 'NoCancer',
                                    tissue='Nasal',
                                    smoking_categories=c('current','ex.less1month', 'ex.1_to_12months', 'ex.more12months'),
                                    confounding='age + batch + sex + smokingStatus + packyears',
                                    donor_group='Clinic',
                                    removeIneligible=TRUE)

# Differential expression between clinic cancer and clinic benign in bronchial tissue
results_bronchialCancerVsNoCancer <- runDeseq(file_list = sampleFiles,
                                    clinical_table = clin_all,
                                    contrast_variable = 'cancer',
                                    ref_condition = 'NoCancer',
                                    tissue='Bronchial',
                                    smoking_categories=c('current','ex.less1month', 'ex.1_to_12months', 'ex.more12months'),
                                    confounding='age + batch + sex + smokingStatus + packyears',
                                    donor_group='Clinic',
                                    removeIneligible=TRUE)


# Select significantly DE genes (clinic vs healthy)
res <- results_healthyVsClinic$res_table
res_sig <- res[which(res$padj < 0.05),] 
res_sig_up <- res_sig[which(res_sig$log2FoldChange >= 0),] # select only significantly UPregulated genes
res_sig_down <- res_sig[which(res_sig$log2FoldChange < 0),] # select only significantly DOWNregulated genes


################################### GO ENRICHMENT ###################################
universe = rownames(res)
goUp <- enrichGO(rownames(res_sig_up),
                         universe=universe,
                         'org.Hs.eg.db',
                         keyType = 'ENSEMBL',
                         ont = "BP",
                         pvalueCutoff = 0.05,
                         pAdjustMethod = "BH",
                         qvalueCutoff = 0.05)
goDwn <- enrichGO(rownames(res_sig_down),
                          universe=universe,
                          'org.Hs.eg.db',
                          keyType = 'ENSEMBL',
                          ont = "BP",
                          pvalueCutoff = 0.05,
                          pAdjustMethod = "BH",
                          qvalueCutoff = 0.05)

################################### VARIANCE COMPONENTS ANALYSIS ###################################
# Load expression data
# this is a list of dataframes generated with the loadExpressionData function (file 3.SmokeInjuryReversibility)
genedat <- readRDS('data/genedat_nasal_and_bronchial.RDS')
# filter out low variance genes
genedat.var = sapply(genedat, function(x){var(x$gxp[x$tissue=='nasal'], na.rm=TRUE)})
genedat = genedat[(genedat.var>0) & (log(genedat.var) > -4)]

# NASAL 

genedat_ns <- lapply(genedat, function(x){
  x <- x[x$tissue=='nasal',]
})
# Extract vst gene expression corrected for batch (residuals)
exprmat_ns = sapply(genedat_ns, function(x){return(setNames(x$gxpnobatch, x$sampleid))})
exprmat_ns <- exprmat_ns[rownames(clin_ns),]

# clinical info table
clin_ns <- clinical[clinical$Tissue=='Nasal',]
clin_br <- clinical[clinical$Tissue=='Bronchial',]
info <- data.frame(Individual= clin_ns$PatientID,
                   Batch=clin_ns$Experiment,
                   Sex= clin_ns$Gender,
                   Age= clin_ns$Age,
                   Packyears= clin_ns$PacksYear,
                   SmokingStatus=clin_ns$SmokingStatus,
                   Cancer=clin_ns$Cancer,
                   HVstatus=ifelse(clin_ns$CancerType=='HV', 'HV', "Clinic"))

# Calculate variance components of data corrected for batch 
# formula including all variables to consider
form <- ~ (1|Age) + (1|Sex) + (1|Packyears) + (1|SmokingStatus) + (1|Cancer) + (1|HVstatus)
# Calculate variance components
varPart_ns <- fitExtractVarPartModel(t(exprmat_ns), form, info)
vp_ns <- sortCols(varPart_ns)

to_plot <- data.frame(Variable=names(colMeans(vp)), Variance=colMeans(vp)*100)
ggplot(to_plot, aes(x=reorder(Variable, Variance), y=Variance, fill=Variable)) +
  geom_bar(stat = 'identity') +
  geom_text(aes(label=round(Variance,digits = 1)), vjust=-0.25, size=6) +
  xlab('Variable') +
  ylab('% Variance explained') +
  theme_minimal() +
  theme(axis.title = element_text(size=20),
        axis.text.x = element_text(size=18, angle = 30),
        axis.text.y = element_text(size=18),
        legend.position = 'none')


# BRONCHIAL
genedat_br <- lapply(genedat, function(x){
  x <- x[x$tissue=='bronchial',]
})
# Extract vst gene expression corrected for batch (residuals)
exprmat_br = sapply(genedat_br, function(x){return(setNames(x$gxpnobatch, x$sampleid))})
exprmat_br <- exprmat_br[rownames(clin_br),]

# clinical info table
genedat_br <- lapply(genedat, function(x){
  x <- x[x$tissue=='bronchial',]
})
# Extract vst gene expression corrected for batch (residuals)
exprmat_br = sapply(genedat_br, function(x){return(setNames(x$gxpnobatch, x$sampleid))})
exprmat_br <- exprmat_br[rownames(clin_br),]

info <- data.frame(Individual= clin_br$PatientID,
                   Batch=clin_br$Experiment,
                   Sex= clin_br$Gender,
                   Age= clin_br$Age,
                   Packyears= clin_br$PacksYear,
                   SmokingStatus=clin_br$SmokingStatus,
                   Cancer=clin_br$Cancer,
                   HVstatus=ifelse(clin_br$CancerType=='HV', 'HV', "Clinic"))

# formula including all variables to consider
form <- ~ (1|Age) + (1|Sex) + (1|Packyears) + (1|SmokingStatus) + (1|Cancer)
# Calculate variance components
varPart_br <- fitExtractVarPartModel(t(exprmat_br), form, info)
vp_br <- sortCols(varPart_br)


# BOTH

exprmat_all = sapply(genedat, function(x){return(setNames(x$gxpnobatch, x$sampleid))})
exprmat_all <- exprmat_all[rownames(clin_all),]

# clin_all info table
info <- data.frame(Individual= clin_all$PatientID,
                   Batch=clin_all$Experiment,
                   Sex= clin_all$Gender,
                   Age= clin_all$Age,
                   Packyears= clin_all$PacksYear,
                   SmokingStatus=clin_all$SmokingStatus,
                   Cancer=clin_all$Cancer,
                   Tissue=clin_all$Tissue,
                   HVstatus=ifelse(clin_all$CancerType=='HV', 'HV', "Clinic"))


# formula including all variables to consider
form <- ~ (1|Age) + (1|Sex) + (1|Packyears) + (1|SmokingStatus) + (1|Cancer) + (1|Tissue) + (1|HVstatus)
# Calculate variance components
varPart_all <- fitExtractVarPartModel(t(exprmat_all), form, info)
vp_all <- sortCols(varPart_all)


to_plot2 <- gather(vp_all, key = 'Variable', value = 'variance_explained')
ggplot(to_plot2, aes(x=variance_explained*100)) +
  geom_histogram(bins = 100, fill='#1F78B4') +
  facet_wrap(~Variable,nrow = 2, scales = 'free') +
  theme_bw() +
  theme(axis.title = element_text(size=20),
        axis.text.x = element_text(size=12),
        axis.text.y = element_text(size=12),
        strip.text = element_text(size=14),
        legend.position = 'none') +
  xlab('% Variance explained')

# Plot only explained variance
vp_to_plot <- as.data.frame(vp_all)
vp_to_plot$explained <- 1-test$Residuals
vp_to_plot <- apply(vp_to_plot,2, function(x){x <- x/vp_to_plot$explained})
vp_to_plot <- vp_to_plot[,1:7]
vp_to_plot <- vp_to_plot[!is.na(rowSums(vp_to_plot)),]
to_plot <- data.frame(Variable=names(colMeans(vp_to_plot)), Variance=colMeans(vp_to_plot)*100)
ggplot(to_plot, aes(x=reorder(Variable, Variance), y=Variance, fill=Variable)) +
  geom_bar(stat = 'identity', fill='#1F78B4') +
  xlab('Variable') +
  ylab('% Explained variance') +
  theme_minimal() +
  theme(axis.title.x = element_text(size=40),
        axis.title.y = element_blank(),
        axis.text.x = element_text(size=38),
        axis.text.y = element_text(size=38),
        legend.position = 'none') + coord_flip()
