################################### FUNCTIONS ###################################

createActivityMatrix <- function(tissue_data, regulon){
  # select genes used in single-gene approach (filtered to exclude invariant and low variance genes)
  geneannot = readRDS('data/geneannot.RDS') # gene ID annotation from biomaRt
  tissue_data <- t(tissue_data[,geneannot$ensembl_gene_id])
  rownames(tissue_data) <- geneannot$hgnc_symbol
  # create null model for viper
  eset <- as.matrix(tissue_data)
  vpsig <- viperSignature(eset, eset, method="mean", per=1000,verbose = TRUE)
  #run viper
  act = viper(eset = vpsig, regulon = regulon, pleiotropy = FALSE, nes = T, method = 'none', minsize = 20, eset.filter = F, verbose = TRUE)
  actmat <- as.data.frame(t(act))
}

# Plot TF over smoking/cancer/... status
calculate_residuals = function(localdat, y, confounding, facet_by, center=FALSE) {
  if (!is.null(facet_by)) { ## split by facet groups
    groups = trimws(strsplit(facet_by, '\\+')[[1]])
    splitdat = split(localdat, localdat[,groups])
    
    splitresid = lapply(splitdat, function(dat) {
      if (nrow(dat)>0) {
        if (!is.null(confounding)) {
          confform = as.formula(paste(y, ' ~ ', confounding))
          if (grepl('\\|', confounding)) { # mixed model
            residuals = resid(lmer(confform, data=dat))
          } else {
            LM = lm(confform, data=dat)
            residuals = resid(LM) 
            if (!center) {
              residuals = residuals + attr(predict(LM, type='terms'), 'constant')
            }
          }
        } else {
          residuals = dat[,y] 
        }
      } else {
        residuals = numeric()
      }
      return(residuals)
    })
    residuals = unsplit(splitresid, localdat[,groups])
  } else { ## no faceting
    if (!is.null(confounding)) {
      confform = as.formula(paste(y, ' ~ ', confounding))
      if (grepl('\\|', confounding)) { # mixed model
        residuals = resid(lmer(confform, data=localdat))
      } else {
        residuals = resid(lm(confform, data=localdat))
      }
    } else {
      residuals = localdat[,y] 
    }
  }
  return(residuals)
}

fix_label = function(retval) {
  retval = gsub('packsyear', 'Pack years', retval)
  retval = gsub('smokingstatus', 'Smoking status', retval)
  retval = gsub('gxp', 'Expression', retval)
  retval = gsub('cancer', 'Cancer', retval)
  retval = gsub('hv', 'donor status', retval)
  return(retval)
}

plot_reg = function(gene, genedat, y='ract', x='smokingstatus', color='packsyear', shape=NULL, confounding=NULL, geneannot=NULL, genestats=NULL, facet_by=NULL, fill_scale=NULL, color_scale=NULL) {
  if (is.data.frame(genedat)) {
    localdat = genedat
  } else if (is.list(genedat)){
    localdat = genedat[[gene]]
  } else {
    stop('Parameter \'regdat\' must be a dataframe or list of dataframes')
  }
  
  localdat$expression = calculate_residuals(localdat, y, confounding, facet_by)
  
  ## fix legend and axis labels
  colorlab = fix_label(color)
  shapelab = fix_label(shape)
  xlab = fix_label(x)
  ylab = fix_label(y)
  
  ## plot
  p = ggplot(localdat, aes(x=get(x), y=expression, fill=get(x))) + 
    geom_boxplot(notch=T) + 
    #    geom_jitter(aes_string(color=color, shape=shape)) +
    #stat_summary(fun.y=mean, geom="point", shape=16, size=2, show.legend = FALSE) + 
    #stat_summary(aes(group=1), fun.y=mean, geom="line", size=0.6, alpha=0.8, linetype='dotted', show.legend = FALSE) +
    guides(fill=FALSE) +
    labs(color=colorlab, shape=shapelab, x=xlab, y=ylab)
  
  
  if (is.null(fill_scale)) {
    if (x=='smokingstatus') {
      fill_scale = createScaleFillSmoking(levels(localdat[,x]))
    } else {
      fill_scale = scale_fill_brewer(palette='Set1')
    }
  }
  p = p + fill_scale
  
  if (!is.null(color_scale)) {
    p = p + color_scale
  }
  
  if (!is.null(facet_by)) {
    facetform = as.formula(paste('. ~', facet_by))
    p = p + facet_wrap(facetform, scales = 'fixed', nrow = 3, strip.position = 'right') 
  }
  
  title = gene
  if (!is.null(geneannot)) {
    hgnc_symbols = trimws(paste(geneannot$hgnc_symbol[geneannot$ensembl_gene_id==gene], collapse=', '))
    if (nchar(hgnc_symbols)==0) {
      hgnc_symbols = '<unknown>'
    }
    title = sprintf('%s - %s', title, hgnc_symbols)
  }
  
  if (!is.null(genestats)) {
    class = genestats[gene,]$geneclass
    if (is.null(class)) class = 'NA'
    post = genestats[gene,]$maxpost
    if (is.null(post)) post = NA
    title = sprintf('%s [%s %.2f]', title, class, post)
  }
  p = p + ggtitle(title)
  return(p)
}

################################### LOAD LIBRARIES ###################################
library(viper)
library(biomaRt)
library(dplyr)
library(igraph)
library(ggplot2)
library(ggpubr)
library(clusterProfiler)

################################### CREATE ACTIVITY MATRIX ################################### 

# load nasal and bronchial networks

# network_NS.3col and network_BR.3col are the output of ARACNe-AP (see run_aracne.sh), without the last column
ns_regulon <- aracne2regulon("data/aracne/network_NS.3col", 
                             "data/expmat_NS.tsv",
                             format = "3col")
saveRDS(ns_regulon, file = 'data/regulons/aracne_nasal_network.RDS')

br_regulon <- aracne2regulon("data/aracne/results/bronchial/network_BR.3col",
                             "data/expmat_BR.tsv",
                             format = "3col")
saveRDS(br_regulon, file = 'data/regulons/aracne_bronchial_network.RDS')

## load nasal expression data
nasal_data <- read.table("data/normalized_counts/exprDataAnnotatedNS.tsv")
## load bronchial expression data
bronchial_data <- read.table("data/normalized_counts/exprDataAnnotatedBR.tsv")

# Run viper on nasal data
aracne_actmat_ns <- createActivityMatrix(nasal_data, ns_regulon)
write.table(aracne_actmat_ns, file = 'data/activity_matrices/aracne_act_matrix_ns.tsv',sep = "\t", quote = F)

# Run viper on bronchial data
aracne_actmat_br <- createActivityMatrix(bronchial_data, br_regulon)
write.table(aracne_actmat_br, file = '/data/activity_matrices/aracne_act_matrix_br.tsv',sep = "\t", quote = F)


################################### IDENTIFY 'CORE' SMOKE INJURY TFs ###################################

# Load smoke injury TFs and smoke injury genes table
geneannot <- readRDS('data/geneannot.RDS') # gene ID annotation from biomaRt
regstats_clinic <- read.table('data/smoke_injury/regstats_CP.tsv', header = T) # smoke injury reversibility classification results from 3.SmokeInjuryReversibility
regstats_healthy <- read.table('data/smoke_injury/regstats_HV.tsv', header = T)
genestats_clinic <- read.table('data/smoke_injury/genestats_Clinic.tsv', header = T)
genestats_healthy <- read.table('data/smoke_injury/genestats_HV.tsv', header = T)
regs <- union(rownames(regstats_clinic[regstats_clinic$geneclass_strict%in%c('RR','SR','IR','CA'),]),
              rownames(regstats_healthy[regstats_healthy$geneclass_strict%in%c('RR','SR','IR','CA'),]))
regs[regs=='NKX3.1'] <- 'NKX3-1'
genes <- union(rownames(genestats_clinic[genestats_clinic$geneclass_strict%in%c('RR','SR','IR','CA'),]),
               rownames(genestats_healthy[genestats_healthy$geneclass_strict%in%c('RR','SR','IR','CA'),]))
genes <- geneannot[genes,]$hgnc_symbol

smokenet <- ns_regulon[regs]

target_overlap <- lapply(smokenet, function(x){
  sum(names(x$tfmode)%in%genes)
})
target_overlap <- unlist(target_overlap)
target_overlap <- sort(target_overlap, decreasing=T)

coreTFs <- names(target_overlap)[1:25]

## Plot network of 'core' smoke injury  TFs
# Functions
# Function to calculate pairwise jaccard coefficient
jaccard<-function(regulon){
  output<-matrix(0,nrow=length(regulon),ncol=length(regulon))
  rownames(output)<-colnames(output)<-names(regulon)
  for(i in 1:length(regulon)){
    edgesA<-names(regulon[[i]]$tfmode)
    for(j in 1:length(regulon)){
      edgesB<-names(regulon[[j]]$tfmode)
      intersection<-length(intersect(edgesA,edgesB))
      union<-length(union(edgesA,edgesB))
      jac<-intersection/union
      output[i,j]<-jac
    }
  }
  return(output)
}
# Jaccard network
matrix2network<-function(x,thr){
  w<-which(x>thr,arr.ind=TRUE)
  # Remove self-loops and reciprocal edges
  w<-w[apply(w,1,function(x){if(x[1]>=x[2]){return(FALSE)}else{return(TRUE)}}),]
  a<-rownames(x)[w[,1]]
  b<-rownames(x)[w[,2]]
  edges<-cbind(a,b)
  jis<-rep(0,nrow(edges))
  for(i in 1:length(jis)){
    jis[i]<-x[a[i],b[i]]
  }
  edges<-cbind(edges,jis)
  return(edges)
}

# Calculate Jaccard coefficients
jaccard_nasal<-jaccard(ns_regulon)
# Calculate regulon sizes
size_nasal<-sapply(ns_regulon,function(x){length(x$tfmode)})
# Jaccard Network
net <- matrix2network(jaccard_nasal,thr=0.2)

# Prepare Cytoscape
#-- The Network Table
network<-data.frame(NodeA=net[,1],NodeB=net[,2],JI=as.numeric(net[,3]))
network <- network[network$NodeA%in%coreTFs & network$NodeB%in%coreTFs,]
#-- The Nodes Table
nodes<-data.frame(
  Node=names(size_nasal),
  RegulonSize=size_nasal,
  stringsAsFactors = F
)
nodes <- nodes[coreTFs,]
nodes$class <- regstats_clinic[coreTFs,]$geneclass_strict

write.table(network,file="cytoscape_smokeMRsNetTable.txt",sep="\t",quote=FALSE,row.names=FALSE)
write.table(nodes,file="cytoscape_smokeMRsNodeTable.txt",sep="\t",quote=FALSE,row.names=FALSE)

################################### PLOT GWAS-ENRICHED TFs ###################################

# Make tables for Cytoscape visualization
# Load bronchial network
brnet <- readRDS('data/regulons/aracne_bronchial_network.RDS')
brnet_aracne <- read.table('data/aracne/network_BR.3col', sep = '\t', header = F)
colnames(brnet_aracne) <- c('Regulator', 'Target', 'MI')
# Read-in list of GWAS genes with different significant thresholds
gwas_genes_relaxed <- readLines('data/GWAS_results/eQTL_of_LD_snps_cov5_OneSampPerPatient_one_gene_per_GWAS_SNP.txt_gene_list_pval0.05')
gwas_genes_strict <- readLines('data/GWAS_results/eQTL_of_LD_snps_cov5_OneSampPerPatient_one_gene_per_GWAS_SNP.txt_gene_list_pval1e-06')
gwas_genes_very_relaxed <- readLines('data/GWAS_results/eQTL_of_LD_snps_cov5_OneSampPerPatient_one_gene_per_GWAS_SNP.txt_gene_list_pval1')

# GO enrichment
universe <- rownames(geneannot)
gwas_go <- enrichGO(gwas_genes_strict,
                    universe=universe,
                    'org.Hs.eg.db',
                    keyType = 'ENSEMBL',
                    ont = "BP",
                    pvalueCutoff = 0.05,
                    pAdjustMethod = "BH",
                    qvalueCutoff = 0.05)
barplot(gwas_go, showCategory=5, x='Count')

gwas_genes_relaxed <- geneannot[gwas_genes_relaxed,]$hgnc_symbol
gwas_genes_strict <- geneannot[gwas_genes_strict,]$hgnc_symbol
gwas_genes_very_relaxed <- geneannot[gwas_genes_very_relaxed,]$hgnc_symbol

selected_TFs <- c('CIITA', 'IRF1', 'SPIB', 'ETV7')
brnet_selected <- brnet_aracne[brnet_aracne$Regulator%in%selected_TFs,]

node_list <- unique(union(brnet_selected$Regulator, brnet_selected$Target))
nodes <- data.frame(Node= node_list,
                    TF= ifelse(node_list%in%brnet_selected$Regulator, 'yes', 'no'),
                    GWAS = ifelse(node_list%in%gwas_genes_strict, 'strict', ifelse(node_list%in%gwas_genes_relaxed, 'relaxed', ifelse(node_list%in%gwas_genes_very_relaxed, 'very_relaxed', 'none'))),
                    stringsAsFactors = F)

# Are genes in network also smoke injury risk genes?
risk_genes <- rownames(genestats_healthy[genestats_healthy$geneclass_strict!=genestats_clinic$geneclass_strict,])
risk_genes_symbol <- geneannot[risk_genes,]$hgnc_symbol
risk_genes_symbol <- risk_genes_symbol[risk_genes_symbol!='']
nodes$risk_gene <- ifelse(nodes$Node%in%risk_genes_symbol, 'yes', 'no')
# reversibility in Clinic
genestats_risk_genes <- genestats_clinic[geneannot[geneannot$hgnc_symbol%in%risk_genes_symbol,]$ensembl_gene_id,]
genestats_risk_genes$geneid <- geneannot[rownames(genestats_risk_genes),]$hgnc_symbol
rownames(genestats_risk_genes) <- genestats_risk_genes$geneid
genestats_risk_genes <- genestats_risk_genes[risk_genes_symbol,]
nodes$class_CP <- NA
nodes[nodes$risk_gene=='yes',]$class_CP <- as.character(genestats_risk_genes[nodes[nodes$risk_gene=='yes',]$Node,]$geneclass_strict)
# reversibility in HV
genestats_risk_genes2 <- genestats_healthy[geneannot[geneannot$hgnc_symbol%in%risk_genes_symbol,]$ensembl_gene_id,]
genestats_risk_genes2$geneid <- geneannot[rownames(genestats_risk_genes2),]$hgnc_symbol
rownames(genestats_risk_genes2) <- genestats_risk_genes2$geneid
genestats_risk_genes2 <- genestats_risk_genes2[risk_genes_symbol,]
nodes$class_HV <- NA
nodes[nodes$risk_gene=='yes',]$class_HV <- as.character(genestats_risk_genes2[nodes[nodes$risk_gene=='yes',]$Node,]$geneclass_strict)

# save input for Cytoscape
write.table(brnet_selected,file="data/GWAS_results/cytoscape_BR_GWAS_netTable",sep="\t",quote=FALSE,row.names=FALSE)
write.table(nodes,file="data/GWAS_results/cytoscape_BR_GWAS_nodeTable",sep="\t",quote=FALSE,row.names=FALSE)

# Plot activity in BR/NS
clinical <- readRDS('data/clinical/full_clinical_table.RDS')
clin_br <- clinical[clinical$Tissue=='Bronchial',]
actmat_BR <- read.table('data/activity_matrices/aracne_act_matrix_br.tsv')
actmat_BR <- cbind(actmat_BR,clin_br)
actmat_BR <- actmat_BR[actmat_BR$CancerType!='Ineligible',]
regulon.columns <- colnames(actmat_BR)[-(1491:1505)]
datreg = melt(actmat_BR, measure.vars=regulon.columns, value.name='ract', variable.name='regulonid')
regdat_br <- split(datreg, datreg$regulonid)

lapply(selected_TFs, function(g){
  plot_reg(g, regdat_br, y='ract', x='Cancer', confounding='Experiment', genestats=NULL, fill_scale=NULL, color_scale=scale_color_brewer(palette='Dark2')) +
    theme_minimal() + theme(strip.text = element_text(size=18),
                            axis.title = element_text(size=20),
                            axis.text.x = element_text(size=18, angle = 45),
                            title = element_text(size=22)) +
    stat_compare_means(method = 't.test', size=9) +
    ylab('Activity - Bronchial tissue') +
    xlab('Disease Status')
})

regdat_ns <- readRDS('data/regulondat_nasal.RDS') # list of dataframes generated by loadActivityData funtion in 3.SmokeInjuryReversibility
regdat_ns <- lapply(regdat_ns, function(x){
  x <- x[clinical[clinical$Tissue=='Nasal',]$CancerType!='Ineligible',]
  x$status <- ifelse(x$hv=='healthy', 'HV', ifelse(x$cancer=='cancer','Clinic cancer', 'Clinic benign'))
  x$status <- factor(x$status, levels = c('HV', 'Clinic benign', 'SP cancer'))
  return(x)
})

my_comparisons <- list( c("HV","Clinic benign"), c("Clinic benign","Clinic cancer"), c("HV","Clinic cancer"))

lapply(selected_TFs, function(g){
  plot_reg(g, regdat_ns, y='ract', x='status', confounding='batch', genestats=NULL, fill_scale=NULL, color_scale=scale_fill_brewer(palette = 'Dark2')) +
    theme_minimal() +
    theme(strip.text = element_text(size=18),
          axis.title = element_text(size=20),
          axis.text.x = element_text(size=18, angle = 45),
          title = element_text(size=22)) +
    stat_compare_means(method = 't.test', comparisons = my_comparisons) +
    ylab('Activity - Nasal tissue') +
    xlab('Disease Status')
})