library(biomaRt)
library(ggplot2)
library(VennDiagram)
library(wesanderson)
library(gridExtra)

args = commandArgs(trailingOnly=TRUE)
if (length(args) < 1 | args[1] == '--help'){
  message("usage: run_meqtl.R snp_genotype.tsv phenotype.tsv covariates.tsv output_directory [snp_positions.tsv phenotype_positions.tsv] [--cisonly]")
  stop()
}


# From the regulon file, extract the target of each TF and store it
TargetHit=function(gene_list,Reg_Network){
  TFs=names(Reg_Network)
  targetHit=c()
  for (i in 1:length(TFs)){
    this_TF=TFs[i]
    targetHit[i]=length(which(gene_list %in% Reg_Network[[this_TF]]))
  }
  res=list("TF"=TFs,
           "TargetHit"=targetHit)
  return(res)
}


overRepTFs=function(list_gene,control_genes,Reg_Network,outfile){
  
  reg_stat=data.frame("TF"=names(Reg_Network),
                      "targetHit"=rep(0,length(names(Reg_Network))),
                      "regSize"=len,stringsAsFactors = F)
  temp=TargetHit(list_gene,Reg_Network = Reg_Network )
  all(reg_stat$TF == temp$TF)
  reg_stat$targetHit=temp$TargetHit
  
  #in case regulons are unheavenly covered by the the control genes
  reg_stat$regSize=TargetHit(control_genes,Reg_Network = Reg_Network )$TargetHit
  overRep=(reg_stat$targetHit/length(list_gene)) / (reg_stat$regSize/(length(control_genes)))
  reg_stat$overRep=overRep
  reg_stat$pval=phyper(q=reg_stat$targetHit,
                       m=reg_stat$regSize,
                       n=length(control_genes)-reg_stat$regSize,
                       k=length(list_gene),lower.tail = F)
  
  reg_stat$padj=p.adjust(reg_stat$pval)
  write.table(file = outfile,reg_stat,sep="\t",quote=F)
  return(reg_stat)
}

# Load metadata: a table summarizing the info for all genes:
#-Is the gene DE between HV and clinic
#-Is the gene a "GWAS" gene
#-What is the reversibility class
# ....
metadata=read.table("gene_summary_table.tsv",head=T,
                    stringsAsFactors = F,sep="\t")

#Load the regulon data:
#This file is generate in script 2.TranscriptionFactorNetwork
reg=readRDS("~/data/regulons/aracne_bronchial_regulon.RDS")

# Reshapping the TF regulon file and converting gene names
names(reg)=gsub(pattern = "_. -.*",replacement = "",x = names(reg))
genes=names(reg)

biomartCacheClear()
mart <- useDataset("hsapiens_gene_ensembl", useMart(host = "http://grch37.ensembl.org","ENSEMBL_MART_ENSEMBL"))
G_list <- getBM( attributes= c("hgnc_symbol","ensembl_gene_id","chromosome_name"),mart= mart)

Reg_Network=list()
#remove duplicates
RM=which(G_list$chromosome_name %in% c(as.character(1:22),"X","Y")==F)
G_list=G_list[-RM,]

# test if there are still duplicates
RM2=which(G_list$hgnc_symbol=="")
G_list2=G_list[-RM2,]
dup2=which(duplicated(G_list2$hgnc_symbol))
dup=which(duplicated(G_list2$ensembl_gene_id))

#Some Manual adjustments
# UGT2A1 is still duplicated, but should be the only one 
G_list[which(G_list$hgnc_symbol=="UGT2A1"),]

#ASB3 is not in there
gene_ens[which(genes=="ASB3")]="ENSG00000270898"

# SCXA and SCXB have been merged and are now SCX, with ENSG00000260428 
G_list=rbind(G_list,c("SCX","ENSG00000260428",8))
G_list$hgnc_symbol[which(G_list$ensembl_gene_id=="ENSG00000270898")]="ASB3"

G_list$hgnc_symbol[13559]="ZSCAN26"
G_list[13560,]
len=c()

for (i in genes){
  gene_ens=G_list$ensembl_gene_id[which(G_list$hgnc_symbol==i)]
  target=c()
  len=c(len,length(gene_ens))
  for (j in names(reg[[i]][["tfmode"]])){
    this_target=G_list$ensembl_gene_id[which(G_list$hgnc_symbol==j)]
    target=c(target,this_target)
  }
  Reg_Network[[gene_ens]]=target
  
}

## A small example to calculate enrichment:

myHits=metadata$ensembl_id[which(metadata$GWAS==TRUE)]
all_evaluated_genes_QTL=metadata$ensembl_id

#Where to store the result:
outfile="~/outfile.txt"
#- List genes are the hits
#- Reg_Network the TF network
#- Control genes is the universe to test against. By default this should be all genes in the vst file
#  but one might want to use a smaller set of genes

top_TF_GWAS=overRepTFs(list_gene = myHits,
                       Reg_Network = Reg_Network,
                       control_genes = metadata$ensembl_id,
                       outfile = outfile)
